//---------------------------------------------------------------------------
#include <vcl\vcl.h>
#pragma hdrstop

//#include "IvoryBitmap.h"
//---------------------------------------------------------------------------
static inline TIvoryBitmap* ValidCtrCheckBitmap()
{
	return new TIvoryBitmap(NULL);
}
//---------------------------------------------------------------------------
__fastcall TIvoryBitmap::TIvoryBitmap(Classes::TComponent* AOwner)
 : TComponent(AOwner), FHandle(NULL), FFileName(""), FResourceName(""),
	FDllHandle(NULL), FDllName(""), FWidth(0), FHeight(0), FOnChange(NULL)
{
}
//---------------------------------------------------------------------------
__fastcall TIvoryBitmap::~TIvoryBitmap()
{
	if(Handle) DeleteObject(Handle); // delete the bitmap object
   if(DllHandle) FreeLibrary(DllHandle); // free the DLL
}
//---------------------------------------------------------------------------
// set a bitmap handle
void __fastcall TIvoryBitmap::SetHandle(HBITMAP handle)
{
	if(Handle != handle) // if a new handle specified
   {
		if(Handle) DeleteObject(Handle); // delete the last bitmap
   	FHandle = handle; // assign the new handle
	   BITMAP bm;
  		GetObject(Handle, sizeof(bm), &bm);
	   FBits = (unsigned char*)bm.bmBits + bm.bmWidthBytes * (bm.bmHeight - 1);
      FWidthBytes = bm.bmWidthBytes;
      FWidth = bm.bmWidth;
      FHeight = bm.bmHeight;
      if(OnChange) OnChange(this);
   }
}
//---------------------------------------------------------------------------
// create an empty bitmap
void __fastcall TIvoryBitmap::CreateEmpty(int width, int height)
{
	HBITMAP hbm = CreateBitmap(width, height, 1, 8, NULL);
   if(hbm)
   {
//      FWidth = width;
//      FHeight = height;
		Handle = hbm;
//      if(OnChange) OnChange(this);
   }
}
//---------------------------------------------------------------------------
// set a bitmap file name and load a bitmap from the file
void __fastcall TIvoryBitmap::SetFileName(const String filename)
{
	if(FileName != filename) // if a new file name specified
   {
   	FFileName = filename; // assign the new file name
      FResourceName = ""; // clear the resource name
     	LoadBitmapFromFile(); // load the bitmap
   }
}
//---------------------------------------------------------------------------
// load a bitmap from a file
void __fastcall TIvoryBitmap::LoadBitmapFromFile()
{
	HBITMAP hbm = (HBITMAP)LoadImage(NULL, FileName.c_str(), IMAGE_BITMAP, 0, 0, LR_LOADFROMFILE | LR_CREATEDIBSECTION);
   if(hbm)
   {
//	   BITMAP bm;
//   	GetObject(hbm, sizeof(bm), &bm);
//		FWidth = bm.bmWidth;
// 	FHeight = bm.bmHeight;
   	Handle = hbm;
   }
}
//---------------------------------------------------------------------------
// set a bitmap resource name and load the bitmap resource
void __fastcall TIvoryBitmap::SetResourceName(const String resname)
{
	if(ResourceName != resname) // if new resource name specified
   {
   	FResourceName = resname; // assign the new resource name
   	FFileName = ""; // clear the file name
     	LoadBitmapFromResource(); // load the bitmap
   }
}
//---------------------------------------------------------------------------
// load a bitmap resource from the application or from a DLL
void __fastcall TIvoryBitmap::LoadBitmapFromResource()
{
	if(DllHandle)
		Handle = (HBITMAP)LoadImage(DllHandle, ResourceName.c_str(), IMAGE_BITMAP, 0, 0, LR_CREATEDIBSECTION);
      // load from a DLL
   else
		Handle = (HBITMAP)LoadImage(GetModuleHandle(NULL), ResourceName.c_str(), IMAGE_BITMAP, 0, 0, LR_CREATEDIBSECTION);
      // load from the application
}
//---------------------------------------------------------------------------
// set a new DLL handle
void __fastcall TIvoryBitmap::SetDllHandle(HANDLE dllhandle)
{
	if(DllHandle != dllhandle) // if new handle specified
   {
   	if(DllHandle) FreeLibrary(DllHandle); // delete the old DLL
		FDllHandle = dllhandle; // assign the new handle
   	if(DllHandle) // if the handle is valid
	   {
   		char fname[MAX_PATH];
	   	if(GetModuleFileName(DllHandle, fname, sizeof(fname)))
		      FDllName = fname;
      	else
	      	FDllName = "";
         // get the DLL's file name
   	}
	   else // if the handle is NULL
   		FDllName = ""; // no DLL name
   }
}
//---------------------------------------------------------------------------
// set a new DLL file name
void __fastcall TIvoryBitmap::SetDllName(const String dllname)
{
	DllHandle = LoadLibrary(dllname.c_str());
}
//---------------------------------------------------------------------------
/*
// set bitmap from an array of colors
void __fastcall TIvoryBitmap::SetPixels(const unsigned char* pixels, int N_pixels)
{
	if(N_pixels <= Width*Height)
   {
		SetBitmapBits(Handle, N_pixels, pixels);
      if(OnChange) OnChange(this);
   }
}
//---------------------------------------------------------------------------
// get the array of colors from a bitmap
void __fastcall TIvoryBitmap::GetPixels(unsigned char* pixels, int N_pixels) const
{
	if(N_pixels <= Width*Height)
		GetBitmapBits(Handle, N_pixels, pixels);
}
*/
//---------------------------------------------------------------------------
void* __fastcall TIvoryBitmap::GetScanLine(int Row) const
{
   return FBits - FWidthBytes * Row;
}
//---------------------------------------------------------------------------

