//---------------------------------------------------------------------------
#include <vcl\vcl.h>
#pragma hdrstop

//#include "IvoryDraw.h"
//---------------------------------------------------------------------------
IDirectDrawPalette * DDLoadPalette(IDirectDraw *pdd, LPCSTR szBitmap);

static inline TIvoryPalette* ValidCtrCheckPalette()
{
	return new TIvoryPalette(NULL);
}
//---------------------------------------------------------------------------
__fastcall TIvoryPalette::TIvoryPalette(Classes::TComponent* AOwner)
 : TComponent(AOwner), FDirectDrawPalette(NULL)
{
}
//---------------------------------------------------------------------------
__fastcall TIvoryPalette::~TIvoryPalette()
{
	if(DirectDrawPalette)
   	TheIvoryDraw->ReleasePalette(this);
}
//---------------------------------------------------------------------------
// release the Direct Draw Palette object (clear it from the memory)
void __fastcall TIvoryPalette::Release()
{
	if(DirectDrawPalette)
   {
   	DirectDrawPalette->Release();
      FDirectDrawPalette = NULL;
   }
}
//---------------------------------------------------------------------------
// set the bitmap file name and loads a palette from a bitmap file
void __fastcall TIvoryPalette::SetFileName(const String name)
{
//	if(TheIvoryDraw == NULL || TheIvoryDraw->DirectDraw == NULL) throw EIvoryError(iecPaletteError, "TIvoryPalette::SetFileName_TheIvoryDraw");
	if(FFileName == name) return;
	FFileName = name;
	if(TheIvoryDraw == NULL || TheIvoryDraw->DirectDraw == NULL) return;
	if(DirectDrawPalette)
      TheIvoryDraw->ReleasePalette(this); // release the old palette
  	CreateFromBitmap(); // create the new palette
   if(DirectDrawPalette)
	   TheIvoryDraw->AddPalette(this);
	if(OnChange) OnChange(this);
}
//---------------------------------------------------------------------------
TColor __fastcall TIvoryPalette::GetColor(int n) const
{
	if(DirectDrawPalette == NULL) throw EIvoryError(iecPaletteError, "TIvoryPalette::GetColor_DirectDrawPalette");
	if(n < 0 || n > 255) throw EIvoryError(iecPaletteError, "TIvoryPalette::GetColor_n");
	PALETTEENTRY ape;
	HRESULT ddrval = DirectDrawPalette->GetEntries(0, n, 1, &ape);
   if(ddrval != DD_OK) throw EIvoryError(ddrval, "TIvoryPalete::GetColor_GetEntries");
   return TColor(RGB(ape.peRed, ape.peGreen, ape.peBlue));
}
//---------------------------------------------------------------------------
// create a palette from a bitmap file
void __fastcall TIvoryPalette::CreateFromBitmap()
{
	bool refresh = false;
	if(TheIvoryDraw->Palette == this) refresh = true; // if this is the active palette, we need to refresh it
	FDirectDrawPalette = DDLoadPalette(TheIvoryDraw->DirectDraw, FileName.c_str());
   if(refresh) TheIvoryDraw->Palette = this;
}
//---------------------------------------------------------------------------
// Create a DirectDraw palette object from a bitmap resoure
// If the resource does not exist or NULL is passed create a default 332 palette
// This function is written by Microsoft
IDirectDrawPalette * DDLoadPalette(IDirectDraw *pdd, LPCSTR szBitmap)
{
    IDirectDrawPalette* ddpal;
    int                 i;
    int                 n;
    int                 fh;
    HRSRC               h;
    LPBITMAPINFOHEADER  lpbi;
    PALETTEENTRY        ape[256];
    RGBQUAD *           prgb;

    //
    // build a 332 palette as the default.
    //
    for (i=0; i<256; i++)
    {
        ape[i].peRed   = (BYTE)(((i >> 5) & 0x07) * 255 / 7);
        ape[i].peGreen = (BYTE)(((i >> 2) & 0x07) * 255 / 7);
        ape[i].peBlue  = (BYTE)(((i >> 0) & 0x03) * 255 / 3);
        ape[i].peFlags = (BYTE)0;
    }

    //
    // get a pointer to the bitmap resource.
    //
    if (szBitmap && (h = FindResource(NULL, szBitmap, RT_BITMAP)) != NULL)
    {
        lpbi = (LPBITMAPINFOHEADER)LockResource(LoadResource(NULL, h));
        if (!lpbi)
            OutputDebugString("lock resource failed\n");
        prgb = (RGBQUAD*)((BYTE*)lpbi + lpbi->biSize);

        if (lpbi == NULL || lpbi->biSize < sizeof(BITMAPINFOHEADER))
            n = 0;
        else if (lpbi->biBitCount > 8)
            n = 0;
        else if (lpbi->biClrUsed == 0)
            n = 1 << lpbi->biBitCount;
        else
            n = lpbi->biClrUsed;

        //
        //  a DIB color table has its colors stored BGR not RGB
        //  so flip them around.
        //
        for(i=0; i<n; i++ )
        {
            ape[i].peRed   = prgb[i].rgbRed;
            ape[i].peGreen = prgb[i].rgbGreen;
            ape[i].peBlue  = prgb[i].rgbBlue;
            ape[i].peFlags = 0;
        }
    }
    else if (szBitmap && (fh = _lopen(szBitmap, OF_READ)) != -1)
    {
        BITMAPFILEHEADER bf;
        BITMAPINFOHEADER bi;

        _lread(fh, &bf, sizeof(bf));
        _lread(fh, &bi, sizeof(bi));
        _lread(fh, ape, sizeof(ape));
        _lclose(fh);

        if (bi.biSize != sizeof(BITMAPINFOHEADER))
            n = 0;
        else if (bi.biBitCount > 8)
            n = 0;
        else if (bi.biClrUsed == 0)
            n = 1 << bi.biBitCount;
        else
            n = bi.biClrUsed;

        //
        //  a DIB color table has its colors stored BGR not RGB
        //  so flip them around.
        //
        for(i=0; i<n; i++ )
        {
            BYTE r = ape[i].peRed;
            ape[i].peRed  = ape[i].peBlue;
            ape[i].peBlue = r;
        /*
            ape[i].peRed =0;
            ape[i].peGreen =0;
            ape[i].peBlue =0;
            ape[i].peFlags =0;
        */
        }
    }

    pdd->CreatePalette(DDPCAPS_8BIT /*| DDPCAPS_ALLOW256*/, ape, &ddpal, NULL);

    return ddpal;
}
//---------------------------------------------------------------------------

